@@ -19,13 +19,12 @@
 #include "panel-globals.h"
 
 
+
 static GdkPixbuf *button_load_pixbuf (const char  *file,
-				      int          preffered_size,
 				      char       **error);
 
 enum {
 	PROP_0,
-	PROP_SIZE,
 	PROP_HAS_ARROW,
 	PROP_ORIENT,
 	PROP_ICON_NAME,
@@ -242,13 +241,9 @@ button_widget_unset_pixbufs (ButtonWidget *button)
 }
 
 static void
-button_widget_load_pixbuf_and_scale (ButtonWidget *button,
-				     int           dest_width,
-				     int           dest_height)
+button_widget_load_pixbuf_and_scale (ButtonWidget *button)
 {
 	double scale;
-	double scale_x;
-	double scale_y;
 	int    width;
 	int    height;
 
@@ -267,8 +262,7 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 		else {
 			char *error = NULL;
 
-			button->pixbuf = button_load_pixbuf (
-						button->filename, button->size, &error);
+			button->pixbuf = button_load_pixbuf (button->filename, &error);
 			if (error) {
 				panel_error_dialog (gdk_screen_get_default (),
 						    "cannot_load_pixbuf",
@@ -284,13 +278,16 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 			return;
 	}
 
+	if (button->size <= 1)
+		return;
+
 	width  = gdk_pixbuf_get_width  (button->pixbuf);
 	height = gdk_pixbuf_get_height (button->pixbuf);
 
-	scale_x = (double) dest_width  / width;
-	scale_y = (double) dest_height / height;
-
-	scale = MIN (scale_x, scale_y);
+	if (button->orientation & PANEL_HORIZONTAL_MASK)
+		scale = (double) button->size / height;
+	else
+		scale = (double) button->size / width;
 
 	width  *= scale;
 	height *= scale;
@@ -310,21 +307,15 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 		g_object_unref (button->scaled_hc);
 	
 	button->scaled_hc = make_hc_pixbuf (button->scaled);
+
+	gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
 static void
 button_widget_reload_pixbuf (ButtonWidget *button)
 {
 	button_widget_unset_pixbufs (button);
-	if (GTK_WIDGET (button)->allocation.width  <= 1 ||
-	    GTK_WIDGET (button)->allocation.height <= 1)
-		return;
-
-	button_widget_load_pixbuf_and_scale (
-		button,
-		GTK_WIDGET (button)->allocation.width,
-		GTK_WIDGET (button)->allocation.height);
-	gtk_widget_queue_draw (GTK_WIDGET (button));
+	button_widget_load_pixbuf_and_scale (button);
 }
 
 static void
@@ -366,9 +357,6 @@ button_widget_get_property (GObject    *object,
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
-	case PROP_SIZE:
-		g_value_set_int (value, button->size);
-		break;
 	case PROP_HAS_ARROW:
 		g_value_set_boolean (value, button->arrow);
 		break;
@@ -403,10 +391,6 @@ button_widget_set_property (GObject      *object,
 		const char *icon_name;
 		const char *stock_id;
 
-	case PROP_SIZE:
-		button->size = g_value_get_int (value);
-		gtk_widget_queue_resize (GTK_WIDGET (button));
-		break;
 	case PROP_HAS_ARROW:
 		button->arrow = g_value_get_boolean (value) ? 1 : 0;
 		gtk_widget_queue_draw (GTK_WIDGET (button));
@@ -502,20 +486,19 @@ load_pixbuf (const char  *file,
 
 static GdkPixbuf *
 button_load_pixbuf (const char  *file,
-		    int          preffered_size,
 		    char       **error)
 {
+/* FIXME: this should be based on the panel size */
+#define PREFERRED_SIZE 48
+
 	GdkPixbuf *retval = NULL;
 	char *tmp;
 
-	if (preffered_size <= 0)
-		preffered_size = 48;
-	
 	if (string_empty (file))
-		return get_missing (preffered_size);
+		return get_missing (PREFERRED_SIZE);
 
 	tmp = g_path_get_basename (file);
-	retval = load_pixbuf (tmp, preffered_size, error);
+	retval = load_pixbuf (tmp, PREFERRED_SIZE, error);
 	g_free (tmp);
 
 	if (!retval && g_path_is_absolute (file)) {
@@ -524,13 +507,15 @@ button_load_pixbuf (const char  *file,
 			*error = NULL;
 		}
 
-		retval = load_pixbuf (file, preffered_size, error);
+		retval = load_pixbuf (file, PREFERRED_SIZE, error);
 	}
 
 	if (!retval)
-		retval = get_missing (preffered_size);
+		retval = get_missing (PREFERRED_SIZE);
 
 	return retval;
+
+#undef PREFERRED_SIZE
 }
 
 #define SCALE(x) (((x)*size)/48.0)
@@ -619,6 +604,9 @@ button_widget_expose (GtkWidget         *widget,
 		pb = button_widget->scaled_hc;
 	else
 		pb = button_widget->scaled;
+
+	if (!pb)
+		return FALSE;
 	
 	w = gdk_pixbuf_get_width (pb);
 	h = gdk_pixbuf_get_height (pb);
@@ -673,7 +661,7 @@ button_widget_expose (GtkWidget         *widget,
 		width = widget->allocation.width -  2 * focus_pad;
 		height = widget->allocation.height - 2 * focus_pad;
 		gtk_paint_focus (widget->style, widget->window,
-				 GTK_WIDGET_STATE (widget),
+				 GTK_STATE_NORMAL,
 				 &event->area, widget, "button",
 				 x, y, width, height);
 	}
@@ -685,28 +673,35 @@ static void
 button_widget_size_request (GtkWidget      *widget,
 			    GtkRequisition *requisition)
 {
-	PanelWidget *panel = PANEL_WIDGET (widget->parent);
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
 
-	requisition->width = requisition->height = panel->sz;
+	if (button_widget->scaled) {
+		requisition->width  = gdk_pixbuf_get_width  (button_widget->scaled);
+		requisition->height = gdk_pixbuf_get_height (button_widget->scaled);
+	}
 }
 
 static void
 button_widget_size_allocate (GtkWidget     *widget,
 			     GtkAllocation *allocation)
 {
-	ButtonWidget *button_widget;
-	GtkButton    *button;
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
+	GtkButton    *button = GTK_BUTTON (widget);
+	int           size;
 
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
+	if (button_widget->orientation & PANEL_HORIZONTAL_MASK)
+		size = allocation->height;
+	else
+		size = allocation->width;
 
-	button_widget = BUTTON_WIDGET (widget);
-	button        = GTK_BUTTON (widget);
+	if (button_widget->size != size) {
+		button_widget->size = size;
+
+		button_widget_load_pixbuf_and_scale (button_widget);
+	}
 
 	widget->allocation = *allocation;
 
-	button_widget_load_pixbuf_and_scale (
-		button_widget, allocation->width, allocation->height);
-
 	if (GTK_WIDGET_REALIZED (widget)) {
 		PanelWidget *panel;
 		int          x, y, w, h;
@@ -811,13 +806,13 @@ button_widget_instance_init (ButtonWidget *button)
 	button->scaled_hc = NULL;
 	
 	button->arrow       = 0;
-	button->size        = -1;
 	button->orientation = PANEL_ORIENTATION_TOP;
 	
 	button->ignore_leave  = FALSE;
 	button->dnd_highlight = FALSE;
 
 	button->pressed_timeout = 0;
+	button->size            = 0;
 
 	panel_signal_connect_object_while_alive (
 			panel_icon_theme, "changed",
@@ -849,15 +844,6 @@ button_widget_class_init (ButtonWidgetClass *klass)
 	button_class->pressed  = button_widget_button_pressed;
 	button_class->released = button_widget_button_released;
 
-	g_object_class_install_property (
-			gobject_class,
-			PROP_SIZE,
-			g_param_spec_int ("size",
-					  _("Size"),
-					  _("The desired ButtonWidget size"),
-					  G_MININT, G_MAXINT, -1,
-					  G_PARAM_READWRITE));
-
 	g_object_class_install_property (
 			gobject_class,
 			PROP_HAS_ARROW,
@@ -923,7 +909,6 @@ button_widget_get_type (void)
 
 GtkWidget *
 button_widget_new (const char       *filename,
-		   int               size,
 		   gboolean          arrow,
 		   PanelOrientation  orientation)
 {
@@ -931,7 +916,6 @@ button_widget_new (const char       *filename,
 
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
-			"size", size,
 			"has-arrow", arrow,
 			"orient", orientation,
 			"icon-name", filename,
@@ -942,7 +926,6 @@ button_widget_new (const char       *filename,
 
 GtkWidget *
 button_widget_new_from_stock (const char       *stock_id,
-			      int               size,
 			      gboolean          arrow,
 			      PanelOrientation  orientation)
 {
@@ -950,7 +933,6 @@ button_widget_new_from_stock (const char       *stock_id,
 
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
-			"size", size,
 			"has-arrow", arrow,
 			"orient", orientation,
 			"stock-id", stock_id,
