@@ -443,10 +443,10 @@ create_launcher (const char *location)
 	launcher->dedit = NULL;
 	launcher->prop_dialog = NULL;
 	launcher->destroy_handler = 0;
+	launcher->non_writable = FALSE;
 
 	/* Icon will be setup later */
 	launcher->button = button_widget_new (NULL /* icon */,
-					      -1,
 					      FALSE,
 					      PANEL_ORIENTATION_TOP);
 
@@ -650,6 +650,8 @@ create_properties_dialog (Launcher  *launcher,
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Launcher Properties"));
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	help = gtk_dialog_add_button (
 			GTK_DIALOG (dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
@@ -662,6 +664,7 @@ create_properties_dialog (Launcher  *launcher,
 			GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
 	launcher->dedit = gnome_ditem_edit_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (launcher->dedit), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    launcher->dedit, TRUE, TRUE, 0);
 
@@ -745,7 +748,7 @@ load_launcher_applet (const char       *location,
 					   GTK_STOCK_PROPERTIES,
 					   _("_Properties"));
 
-	panel_applet_add_callback (launcher->info, "help", GTK_STOCK_HELP, _("_Help"));
+	panel_widget_set_applet_expandable (panel, GTK_WIDGET (launcher->button), FALSE, TRUE);
 
 	/* setup button according to ditem */
 	setup_button (launcher);
@@ -765,6 +768,15 @@ panel_launcher_ensure_hoarded (Launcher   *launcher,
 
 	if (!strchr (launcher_location, G_DIR_SEPARATOR))
 		return; /* already hoarded */
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
+	if ( ! gconf_client_key_is_writable (client, key, NULL)) {
+		/* can't hoard */
+		launcher->non_writable = TRUE;
+		return;
+	}
 	
 	if (!(new_location = panel_launcher_get_filename (launcher_location))) {
 		const char *path;
@@ -776,13 +788,12 @@ panel_launcher_ensure_hoarded (Launcher   *launcher,
 		new_location = panel_launcher_get_filename (path);
 	}
 
-	if (!new_location)
+	if (!new_location) {
+		/* hoarding must have failed */
+		launcher->non_writable = TRUE;
 		return;
+	}
 
-	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
-
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
 	gconf_client_set_string (client, key, new_location, NULL);
 }
 
@@ -819,9 +830,18 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 					 TRUE,
 					 id);
 
-	if (launcher)
+	if (launcher) {
 		panel_launcher_ensure_hoarded (launcher, launcher_location, id);
 
+		if (launcher->non_writable) {
+			AppletUserMenu *menu;
+			menu = panel_applet_get_callback (launcher->info->user_menu,
+							  "properties");
+			if (menu != NULL)
+				menu->sensitive = FALSE;
+		}
+	}
+
 	g_free (launcher_location);
 }
 
@@ -877,11 +897,14 @@ ask_about_launcher (const char  *file,
 				NULL);
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"create_launcher", "Panel");
 	
 	dee = GNOME_DITEM_EDIT (gnome_ditem_edit_new ());
+	gtk_container_set_border_width (GTK_CONTAINER (dee), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    GTK_WIDGET (dee),
 			    TRUE, TRUE, GNOME_PAD_SMALL);
