@@ -737,8 +737,6 @@ panel_make_sure_menu_within_screen (GtkMenu *menu)
 
 	menu->position_func = menu_on_screen;
 	menu->position_func_data = repo;
-
-	our_gtk_menu_position (menu);
 }
 
 static void
@@ -1511,6 +1509,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 
 		if (sim->type == 1) {
 			char *tmp;
+			gboolean objects_writable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
 
 			ii = gnome_desktop_item_new_from_uri (sim->item_loc, 0, NULL);
 
@@ -1528,6 +1527,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 			g_signal_connect (menuitem, "activate",
 					  G_CALLBACK (add_app_to_panel),
 					  sim);
+			gtk_widget_set_sensitive (menuitem, objects_writable);
 
 			menuitem = gtk_image_menu_item_new ();
 			setup_menuitem (menuitem, panel_menu_icon_get_size (),
@@ -1601,6 +1601,8 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 		
 		if (sim->mf != NULL) {
 			GtkWidget *submenu;
+			gboolean objects_writable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
+			gboolean toplevels_writable = panel_profile_list_is_writable (PANEL_GCONF_TOPLEVELS);
 
 			if (sim->type == 0) {
 				submenu = sim->menu;
@@ -1626,6 +1628,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 			g_signal_connect (G_OBJECT(menuitem), "activate",
 				   G_CALLBACK(add_menudrawer_to_panel),
 				   sim->mf);
+			gtk_widget_set_sensitive (menuitem, objects_writable && toplevels_writable);
 
 			menuitem = gtk_image_menu_item_new ();
 			setup_menuitem (menuitem, panel_menu_icon_get_size (),
@@ -1634,6 +1637,7 @@ show_item_menu (GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 			g_signal_connect (G_OBJECT(menuitem), "activate",
 					   G_CALLBACK(add_menu_to_panel),
 					   sim->mf->menudir);
+			gtk_widget_set_sensitive (menuitem, objects_writable);
 
 			menuitem = gtk_image_menu_item_new ();
 			setup_menuitem (menuitem, panel_menu_icon_get_size (),
@@ -2453,7 +2457,9 @@ applet_menu_get_category_icon (const char *untranslated_category)
 
 static GtkWidget *
 applet_menu_append (GtkWidget  *menu,
-		    AppletMenuInfo *applet)
+		    AppletMenuInfo *applet,
+		    gboolean applets_writable,
+		    gboolean objects_writable)
 {
 	GtkWidget *menuitem;
 
@@ -2473,16 +2479,22 @@ applet_menu_append (GtkWidget  *menu,
 		gtk_tooltips_set_tip (panel_tooltips, menuitem, applet->description, NULL);
 	
 	if (applet->action_type != PANEL_ACTION_NONE) {
-		setup_internal_applet_drag (menuitem, applet->drag_data);
+		/* only allow dragging objects if we can add objects */
+		if (panel_profile_list_is_writable (PANEL_GCONF_OBJECTS))
+			setup_internal_applet_drag (menuitem, applet->drag_data);
 
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (add_action_button_to_panel),
 				  GINT_TO_POINTER (applet->action_type));
+		gtk_widget_set_sensitive (menuitem, objects_writable);
 	} else {
-		setup_applet_drag (menuitem, applet->drag_data);
+		/* only allow dragging applets if we can add applets */
+		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+			setup_applet_drag (menuitem, applet->drag_data);
 		
 		g_signal_connect_data (menuitem, "activate", G_CALLBACK (add_bonobo_applet),
 				       g_strdup (applet->drag_data), (GClosureNotify) g_free, 0);
+		gtk_widget_set_sensitive (menuitem, applets_writable);
 	}
 
 	return menuitem;
@@ -2708,10 +2720,15 @@ create_applets_menu (GtkWidget             *menu,
 	GSList            *langs_gslist;
 	GSList            *applets = NULL;
 	GSList		  *item = NULL;
+	gboolean           applets_writable;
+	gboolean           objects_writable;
 
 	if (!applet_list)
 		return NULL;
 
+	applets_writable = panel_profile_list_is_writable (PANEL_GCONF_APPLETS);
+	objects_writable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
+
 	if (!menu)
 		menu = menu_new ();
 
@@ -2774,7 +2791,9 @@ create_applets_menu (GtkWidget             *menu,
 		item = item->next;
 		
 		if (string_empty (applet->category)) {
-			applet_menu_append (menu, applet);
+			applet_menu_append (menu, applet,
+					    applets_writable,
+					    objects_writable);
 			continue;
 		}
 
@@ -2791,7 +2810,9 @@ create_applets_menu (GtkWidget             *menu,
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), prev_menu);
 		}
 
-		applet_menu_append (prev_menu, applet);
+		applet_menu_append (prev_menu, applet,
+				    applets_writable,
+				    objects_writable);
 		
 		if (!applet->static_data)
 			g_free (applet);
@@ -2960,15 +2981,20 @@ setup_remove_this_panel (GtkWidget *menu,
 {
 	PanelWidget *panel_widget;
 	GtkWidget   *label;
+	gboolean     toplevels_writable;
 
 	panel_widget = menu_get_panel (menu);
 
 	g_assert (PANEL_IS_TOPLEVEL (panel_widget->toplevel));
 
+	toplevels_writable = panel_profile_list_is_writable (PANEL_GCONF_TOPLEVELS);
+
 	if (panel_toplevel_is_last_unattached (panel_widget->toplevel))
 		gtk_widget_set_sensitive(menuitem, FALSE);
 	else
-		gtk_widget_set_sensitive(menuitem, TRUE);
+		/* this can only be sensitive if we can actually write to
+		   the toplevels list */
+		gtk_widget_set_sensitive(menuitem, toplevels_writable);
 
 	label = GTK_BIN(menuitem)->child;
 	if(GTK_IS_BOX(label)) {
@@ -3005,6 +3031,7 @@ make_panel_submenu (PanelWidget *panel_widget,
 	Bonobo_ServerInfoList *applet_list;
 	GtkWidget             *menuitem, *submenu;
 
+
 	menuitem = gtk_image_menu_item_new ();
 	setup_menuitem (menuitem,
 			GTK_ICON_SIZE_MENU,
@@ -3018,7 +3045,15 @@ make_panel_submenu (PanelWidget *panel_widget,
 
 	applet_list = instrument_add_submenu_for_reload (GTK_MENU_ITEM (menuitem), submenu);
 
-	make_add_submenu (submenu, applet_list);
+	if ( ! panel_profile_list_is_writable (PANEL_GCONF_APPLETS) &&
+	     ! panel_profile_list_is_writable (PANEL_GCONF_OBJECTS)) {
+		/* if we can't write neither applets nor objects, just
+		   completely ignore this and make the whole submenu
+		   insensitive */
+		gtk_widget_set_sensitive (menuitem, FALSE);
+	} else {
+		make_add_submenu (submenu, applet_list);
+	}
 
 	menuitem = gtk_image_menu_item_new ();
 
@@ -3057,6 +3092,10 @@ make_panel_submenu (PanelWidget *panel_widget,
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (create_new_panel), 
 			  NULL);
+	/* this can only be sensitive if we can actually write to the
+	   toplevels list */
+	gtk_widget_set_sensitive (menuitem, 
+				  panel_profile_list_is_writable (PANEL_GCONF_TOPLEVELS));
 
 	add_menu_separator (menu);
 }
@@ -3141,10 +3180,13 @@ make_add_submenu (GtkWidget             *menu,
 		  Bonobo_ServerInfoList *applet_list)
 {
 	GtkWidget *menuitem, *m;
+	gboolean objects_writable;
+
+	objects_writable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
 
 	/* Add Menu */
 
-	m = create_applets_menu (menu, applet_list);
+	create_applets_menu (menu, applet_list);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_stock_menu_item (
@@ -3153,6 +3195,7 @@ make_add_submenu (GtkWidget             *menu,
 	g_signal_connect (G_OBJECT(menuitem), "activate",
 			   G_CALLBACK(ask_about_launcher_cb),NULL);
 	setup_internal_applet_drag(menuitem, "LAUNCHER:ASK");
+	gtk_widget_set_sensitive (menuitem, objects_writable);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_stock_menu_item (
@@ -3162,6 +3205,7 @@ make_add_submenu (GtkWidget             *menu,
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), m);
 	g_signal_connect (G_OBJECT (m),"show",
 			  G_CALLBACK (submenu_to_display), NULL);
+	gtk_widget_set_sensitive (menuitem, objects_writable);
 
   	menuitem = gtk_image_menu_item_new ();
 	setup_stock_menu_item (
@@ -3171,6 +3215,7 @@ make_add_submenu (GtkWidget             *menu,
 			   G_CALLBACK(add_menu_to_panel),
 			   NULL);
 	setup_internal_applet_drag(menuitem, "MENU:MAIN");
+	gtk_widget_set_sensitive (menuitem, objects_writable);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_stock_menu_item (
@@ -3179,6 +3224,7 @@ make_add_submenu (GtkWidget             *menu,
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (add_menu_bar_to_panel), NULL);
 	setup_internal_applet_drag (menuitem, "MENUBAR:NEW");
+	gtk_widget_set_sensitive (menuitem, objects_writable);
 
 	menuitem = gtk_image_menu_item_new ();
 	setup_stock_menu_item (
@@ -3188,6 +3234,7 @@ make_add_submenu (GtkWidget             *menu,
 			   (GtkSignalFunc) add_drawer_to_panel,
 			   NULL);
 	setup_internal_applet_drag(menuitem, "DRAWER:NEW");
+	gtk_widget_set_sensitive (menuitem, objects_writable);
 }
 
 static GtkWidget *
@@ -3414,7 +3461,7 @@ create_root_menu (GtkWidget   *root_menu,
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
 			menuitem, panel_menu_icon_get_size (),
-			PANEL_STOCK_SCREENSHOT, _("Screenshot..."));
+			PANEL_STOCK_SCREENSHOT, _("Take Screenshot..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (root_menu), menuitem);
 		gtk_tooltips_set_tip (panel_tooltips, menuitem,
 				      _("Take a screenshot of your desktop"),
