@@ -338,6 +338,17 @@ panel_profile_get_toplevel_key (PanelToplevel *toplevel,
 	return panel_gconf_full_key (PANEL_GCONF_TOPLEVELS, current_profile, id, key);
 }
 
+#define TOPLEVEL_IS_WRITABLE_FUNC(k, p, s)                            \
+	gboolean                                                      \
+	panel_profile_is_writable_##p##_##s (PanelToplevel *toplevel) \
+	{                                                             \
+		GConfClient *client;                                  \
+		const char  *key;                                     \
+		client = panel_gconf_get_client ();                   \
+		key = panel_profile_get_toplevel_key (toplevel, k);   \
+		return gconf_client_key_is_writable (client, key, NULL); \
+	}
+
 void
 panel_profile_set_background_type (PanelToplevel       *toplevel,
 				   PanelBackgroundType  background_type)
@@ -375,6 +386,8 @@ panel_profile_get_background_type (PanelToplevel *toplevel)
 	return background_type;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/type", background, type)
+
 void
 panel_profile_set_background_color (PanelToplevel *toplevel,
 				    PanelColor    *color)
@@ -405,6 +418,8 @@ panel_profile_get_background_color (PanelToplevel *toplevel,
 	color->alpha     = opacity;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/color", background, color)
+
 void
 panel_profile_set_background_pango_color (PanelToplevel *toplevel,
 					  PangoColor    *pango_color)
@@ -475,6 +490,8 @@ panel_profile_get_background_opacity (PanelToplevel *toplevel)
 	return opacity;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/opacity", background, opacity)
+
 void
 panel_profile_set_background_image (PanelToplevel *toplevel,
 				    const char    *image)
@@ -507,6 +524,8 @@ panel_profile_get_background_image (PanelToplevel *toplevel)
 	return retval;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/image", background, image)
+
 void
 panel_profile_set_toplevel_name (PanelToplevel *toplevel,
 				 const char    *name)
@@ -539,6 +558,8 @@ panel_profile_get_toplevel_name (PanelToplevel *toplevel)
 	return retval;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("name", toplevel, name)
+
 void
 panel_profile_set_toplevel_orientation (PanelToplevel    *toplevel,
 					PanelOrientation  orientation)
@@ -576,6 +597,8 @@ panel_profile_get_toplevel_orientation (PanelToplevel *toplevel)
 	return orientation;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("orientation", toplevel, orientation)
+
 #define TOPLEVEL_GET_SET_FUNCS(k, p, t, s, a)                         \
 	void                                                          \
 	panel_profile_set_##p##_##s (PanelToplevel *toplevel, a s)    \
@@ -596,7 +619,8 @@ panel_profile_get_toplevel_orientation (PanelToplevel *toplevel)
 		key = panel_profile_get_toplevel_key (toplevel, k);   \
 		retval = gconf_client_get_##t (client, key, NULL);    \
 		return retval;                                        \
-	}
+	}                                                             \
+        TOPLEVEL_IS_WRITABLE_FUNC(k, p, s)
 
 TOPLEVEL_GET_SET_FUNCS ("size",               toplevel,   int,  size,           int)
 TOPLEVEL_GET_SET_FUNCS ("expand",             toplevel,   bool, expand,         gboolean)
@@ -1175,6 +1199,21 @@ panel_profile_remove_from_list (PanelGConfKeyType  type,
 	g_slist_free (list);
 }
 
+gboolean
+panel_profile_list_is_writable (PanelGConfKeyType type)
+{
+	GConfClient *client;
+	const char  *key;
+	const char  *id_list;
+
+	client = panel_gconf_get_client ();
+
+	id_list = panel_gconf_key_type_to_id_list (type);
+
+	key = panel_gconf_general_key (current_profile, id_list);
+	return gconf_client_key_is_writable (client, key, NULL);
+}
+
 void
 panel_profile_create_toplevel (void)
 {
@@ -2171,3 +2210,61 @@ panel_profile_set_show_program_list (gboolean show_program_list)
 	key = panel_gconf_general_key (current_profile, "show_program_list");
 	gconf_client_set_bool (client, key, show_program_list, NULL);
 }
+
+gboolean
+panel_profile_is_writable_show_program_list (void)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_gconf_general_key (current_profile, "show_program_list");
+	return gconf_client_key_is_writable (client, key, NULL);
+}
+
+/* True if all the move keys are writable.  This is kind of
+   a blanket statement.  For example we may only lock one of them
+   down and that would prevent the user from moving the panel at
+   all, but really, making this truly fine grained is probably too
+   much crack */
+gboolean
+panel_profile_can_be_moved_freely (PanelToplevel *toplevel)
+{
+	const char *key;
+	GConfClient *client;
+	
+	if ( ! panel_profile_is_writable_toplevel_orientation (toplevel))
+		return FALSE;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_toplevel_key (toplevel, "screen");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "monitor");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	/* For expanded panels we don't really have to check 
+	   x and y */
+	if (panel_toplevel_get_expand (toplevel))
+		return TRUE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "x");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_profile_get_toplevel_key (toplevel, "x_centered");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "y");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_profile_get_toplevel_key (toplevel, "y_centered");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	return TRUE;
+}
