@@ -58,24 +58,24 @@ struct _PanelAppletPrivate {
 	BonoboPropertyBag          *prop_sack;
 	BonoboItemHandler          *item_handler;
 
-	gchar                      *iid;
+	char                       *iid;
 	GClosure                   *closure;
 	gboolean                    bound;
-	gchar                      *prefs_key;
+	char                       *prefs_key;
 
 	PanelAppletFlags            flags;
 	PanelAppletOrient           orient;
 	guint                       size;
-	gchar                      *background;
+	char                       *background;
 
 	GdkPixmap                  *bg_pixmap;
 
         int                        *size_hints;
         int                         size_hints_len;
 
-	gboolean		    moving_focus_out;
-	gint		   	    focusable_child;
-	guint			    hierarchy_changed_id;
+	gboolean                    moving_focus_out;
+	int                         focusable_child;
+	guint                       hierarchy_changed_id;
 };
 
 static GObjectClass *parent_class;
@@ -153,9 +153,12 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 		gchar *subdir = l->data;
 		gchar *prefs_subdir;
 		gchar *schema_subdir;
+		gchar *tmp;
 
-		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, subdir);
-		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, subdir);
+		tmp = g_path_get_basename (subdir);
+
+		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, tmp);
+		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, tmp);
 
 		panel_applet_associate_schemas_in_dir (
 			client, prefs_subdir, schema_subdir, error);
@@ -163,6 +166,7 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 		g_free (prefs_subdir);
 		g_free (schema_subdir);
 		g_free (subdir);
+		g_free (tmp);
 
 		if (*error) {
 			g_slist_free (list);
@@ -183,7 +187,10 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	GError       *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
-	g_return_if_fail (applet->priv->prefs_key != NULL);
+	g_return_if_fail (schema_dir != NULL);
+
+	if (!applet->priv->prefs_key)
+		return;
 
 	if (opt_error)
 		error = opt_error;
@@ -230,7 +237,7 @@ panel_applet_set_flags (PanelApplet      *applet,
 {
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	applet->priv->flags = flags;
+	bonobo_pbclient_set_short (BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_FLAGS, flags, NULL);
 }
 
 void
@@ -239,24 +246,30 @@ panel_applet_set_size_hints (PanelApplet      *applet,
 			     int               n_elements,
 			     int               base_size)
 {
-	int i;
-	applet->priv->size_hints = g_realloc (applet->priv->size_hints, n_elements * sizeof (int));
-	memcpy (applet->priv->size_hints, size_hints, n_elements * sizeof (int));
-	applet->priv->size_hints_len = n_elements;
+	CORBA_sequence_CORBA_long *seq;
+	CORBA_any                  any;
+	int                        i;
+
+	seq = CORBA_sequence_CORBA_long__alloc ();
+	seq->_length = seq->_maximum = n_elements;
+	seq->_release = CORBA_TRUE;
+	seq->_buffer  = CORBA_sequence_CORBA_long_allocbuf (seq->_length);
 
 	for (i = 0; i < n_elements; i++)
-		applet->priv->size_hints[i] += base_size;
+		seq->_buffer [i] = size_hints [i] + base_size;
+
+	any._type    = TC_CORBA_sequence_CORBA_long;
+	any._release = CORBA_FALSE;
+	any._value   = seq;
+
+	Bonobo_PropertyBag_setValue (BONOBO_OBJREF (applet->priv->prop_sack),
+				     PROPERTY_SIZE_HINTS,
+				     &any,
+				     NULL);
+
+	CORBA_free (seq);
 }
 
-/**
- * panel_applet_get_size:
- * @applet: A #PanelApplet.
- *
- * Returns the current size of the panel which
- * the applet is contained in.
- *
- * Return value: a #guint value.
- */
 guint
 panel_applet_get_size (PanelApplet *applet)
 {
@@ -265,15 +278,6 @@ panel_applet_get_size (PanelApplet *applet)
 	return applet->priv->size;
 }
 
-/**
- * panel_applet_get_orient
- * @applet: A #PanelApplet.
- *
- * Returns the current orientation of the panel which
- * the applet is contained in.
- *
- * Return value: a #PanelAppletOrient value.
- */
 PanelAppletOrient
 panel_applet_get_orient (PanelApplet *applet)
 {
@@ -282,17 +286,6 @@ panel_applet_get_orient (PanelApplet *applet)
 	return applet->priv->orient;
 }
 
-/**
- * panel_applet_setup_menu:
- * @applet: A #PanelApplet.
- * @xml: The xml character string describing the popup menu.
- * @verb_list: The list of #BonoboUIVerbs for the menu.
- * @user_data: The user data pointer for the menu.
- *
- * Sets up a popup menu for @applet described by the xml 
- * string, @xml. See <link linkend="applet-writing">Applet Writing
- * </link> section for a description of the format of the xml.
- */
 void
 panel_applet_setup_menu (PanelApplet        *applet,
 			 const gchar        *xml,
@@ -313,19 +306,6 @@ panel_applet_setup_menu (PanelApplet        *applet,
 	bonobo_ui_component_add_verb_list_with_data (popup_component, verb_list, user_data);
 }
 
-/**
- * panel_applet_setup_menu_from_file:
- * @applet: A #PanelApplet.
- * @opt_datadir: The data directory - i.e. ${prefix}/share (optional).
- * @file: The file's name.
- * @opt_app_name: The application's name (optional).
- * @verb_list: The list of #BonoboUIVerbs for the menu.
- * @user_data: The user data pointer for the menu.
- *
- * Sets up a popup menu for @applet described by the xml 
- * file, @file. See &applet-writing for a description of
- * the format of the xml.
- */
 void
 panel_applet_setup_menu_from_file (PanelApplet        *applet, 
 				   const gchar        *opt_datadir,
@@ -356,14 +336,6 @@ panel_applet_setup_menu_from_file (PanelApplet        *applet,
 		g_free (app_name);
 }
 
-/**
- * panel_applet_get_control:
- * @applet: A #PanelApplet.
- *
- * Retrieves the #BonoboControl associated with @applet.
- *
- * Return value: A #BonobControl on success, %NULL on failure.
- */
 BonoboControl *
 panel_applet_get_control (PanelApplet *applet)
 {
@@ -372,15 +344,6 @@ panel_applet_get_control (PanelApplet *applet)
 	return applet->priv->control;
 }
 
-/**
- * panel_applet_get_popup_component:
- * @applet: A #PanelApplet.
- *
- * Retrieves the #BonoboUIComponent used for popup menus associated
- * with @applet. 
- *
- * Return value: A #BonoboUIComponent on success, or %NULL on failure.
- */
 BonoboUIComponent *
 panel_applet_get_popup_component (PanelApplet *applet)
 {
@@ -398,6 +361,7 @@ panel_applet_finalize (GObject *object)
 		bonobo_object_unref (
 			BONOBO_OBJECT (applet->priv->prop_sack));
 
+	g_free (applet->priv->size_hints);
 	g_free (applet->priv->prefs_key);
 	g_free (applet->priv->background);
 	g_free (applet->priv->iid);
@@ -794,7 +758,7 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 			 int              x,
 			 int              y)
 {
-	GdkPixmap *pixmap;
+	GdkWindow *window;
 	GdkPixmap *retval;
 	GdkGC     *gc;
 	int        width;
@@ -802,13 +766,16 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
-	pixmap = gdk_pixmap_lookup (xid);
-	if (pixmap)
-		g_object_ref (G_OBJECT (pixmap));
-	else
-		pixmap = gdk_pixmap_foreign_new (xid);
+	if (!GTK_WIDGET_REALIZED (applet))
+		return NULL;
 
-	g_return_val_if_fail (pixmap != NULL, NULL);
+	window = gdk_window_lookup (xid);
+	if (window)
+		g_object_ref (window);
+	else
+		window = gdk_window_foreign_new (xid);
+
+	g_return_val_if_fail (window != NULL, NULL);
 
 	gdk_drawable_get_size (GDK_DRAWABLE (GTK_WIDGET (applet)->window),
 			       &width, &height);
@@ -821,13 +788,13 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 
 	gdk_draw_drawable (GDK_DRAWABLE (retval),
 			   gc, 
-			   GDK_DRAWABLE (pixmap),
+			   GDK_DRAWABLE (window),
 			   x, y,
 			   0, 0,
 			   width, height);
 
 	g_object_unref (gc);
-	g_object_unref (pixmap);
+	g_object_unref (window);
 
 	return retval;
 }
@@ -842,7 +809,7 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 
 	retval = PANEL_NO_BACKGROUND;
 
-	if (!applet->priv->background)
+	if (!GTK_WIDGET_REALIZED (applet) || !applet->priv->background)
 		return retval;
 
 	elements = g_strsplit (applet->priv->background, ":", -1);
@@ -892,23 +859,6 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 	return retval;
 }
 
-
-/**
- * panel_applet_get_background
- * @applet: A #PanelApplet.
- * @color: A #GdkColor to be filled in.
- * @pixmap: Returned #GdkPixmap
- *
- * Returns the current background type. If the background
- * type is %PANEL_NO_BACKGROUND both @color and @pixmap will
- * be unaffected. If the background type is %PANEL_COLOR_BACKGROUND
- * then @color will contain the current panel background colour.
- * If the background type is %PANEL_PIXMAP_BACKGROUND, @pixmap will
- * contain a pointer to a #GdkPixmap which is a copy of the applet's
- * portion of the panel's background pixmap.
- * 
- * Return value: a #PanelAppletOrient value.
- */
 PanelAppletBackgroundType
 panel_applet_get_background (PanelApplet *applet,
 			     GdkColor *color,
@@ -958,8 +908,41 @@ panel_applet_get_prop (BonoboPropertyBag *sack,
 		seq->_release = CORBA_TRUE;
 		
 		for (i = 0; i < applet->priv->size_hints_len; i++)
-			seq->_buffer [i] = applet->priv->size_hints[i];
-		}
+			seq->_buffer [i] = applet->priv->size_hints [i];
+	}
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
+
+static void
+panel_applet_handle_background (PanelApplet *applet)
+{
+	PanelAppletBackgroundType  type;
+	GdkColor                   color;
+	GdkPixmap                 *pixmap = NULL;
+
+	type = panel_applet_handle_background_string (applet, &color, &pixmap);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		g_signal_emit (G_OBJECT (applet),
+			       panel_applet_signals [CHANGE_BACKGROUND],
+			       0, PANEL_NO_BACKGROUND, NULL, NULL);
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		g_signal_emit (G_OBJECT (applet),
+			       panel_applet_signals [CHANGE_BACKGROUND],
+			       0, PANEL_COLOR_BACKGROUND, &color, NULL);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		g_signal_emit (G_OBJECT (applet),
+			       panel_applet_signals [CHANGE_BACKGROUND],
+			       0, PANEL_PIXMAP_BACKGROUND, NULL, pixmap);
+
+		g_object_unref (pixmap);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -989,7 +972,7 @@ panel_applet_set_prop (BonoboPropertyBag *sack,
 				       panel_applet_signals [CHANGE_ORIENT],
 				       0, orient);
 		}
-		}
+	}
 		break;
 	case PROPERTY_SIZE_IDX: {
 		guint size;
@@ -1003,44 +986,30 @@ panel_applet_set_prop (BonoboPropertyBag *sack,
                                        panel_applet_signals [CHANGE_SIZE],
                                        0, size);
 		}
-		}
+	}
 		break;
-	case PROPERTY_BACKGROUND_IDX: {
-		PanelAppletBackgroundType  type;
-		GdkColor                   color;
-		GdkPixmap                 *pixmap = NULL;
-
-
+	case PROPERTY_BACKGROUND_IDX:
 		if (applet->priv->background)
 			g_free (applet->priv->background);
 
 		applet->priv->background = g_strdup (BONOBO_ARG_GET_STRING (arg));
 
-		type = panel_applet_handle_background_string (applet, &color, &pixmap);
+		panel_applet_handle_background (applet);
+		break;
+	case PROPERTY_FLAGS_IDX:
+		applet->priv->flags = BONOBO_ARG_GET_SHORT (arg);
+		break;
+	case PROPERTY_SIZE_HINTS_IDX: {
+		CORBA_sequence_CORBA_long *seq = arg->_value;
+		int                        i;
 
-		switch (type) {
-		case PANEL_NO_BACKGROUND:
-			g_signal_emit (G_OBJECT (applet),
-				       panel_applet_signals [CHANGE_BACKGROUND],
-				       0, PANEL_NO_BACKGROUND, NULL, NULL);
-			break;
-		case PANEL_COLOR_BACKGROUND:
-			g_signal_emit (G_OBJECT (applet),
-				       panel_applet_signals [CHANGE_BACKGROUND],
-				       0, PANEL_COLOR_BACKGROUND, &color, NULL);
-			break;
-		case PANEL_PIXMAP_BACKGROUND:
-			g_signal_emit (G_OBJECT (applet),
-				       panel_applet_signals [CHANGE_BACKGROUND],
-				       0, PANEL_PIXMAP_BACKGROUND, NULL, pixmap);
-
-			g_object_unref (pixmap);
-			break;
-		default:
-			g_assert_not_reached ();
-			break;
-		}
-		}
+		applet->priv->size_hints = g_realloc (applet->priv->size_hints,
+						      seq->_length * sizeof (int));
+		for (i = 0; i < seq->_length; i++)
+			applet->priv->size_hints [i] = seq->_buffer [i];
+		
+		applet->priv->size_hints_len = seq->_length;;
+	}
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1100,6 +1069,15 @@ panel_applet_property_bag (PanelApplet *applet)
 	return sack;
 }
 
+static void
+panel_applet_realize (GtkWidget *widget)
+{
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+
+	if (PANEL_APPLET (widget)->priv->background)
+		panel_applet_handle_background (PANEL_APPLET (widget));
+}
+
 static void
 panel_applet_control_bound (BonoboControl *control,
 			    PanelApplet   *applet)
@@ -1117,7 +1095,7 @@ panel_applet_control_bound (BonoboControl *control,
 			       G_TYPE_BOOLEAN, &ret,
 			       PANEL_TYPE_APPLET, applet,
 			       G_TYPE_STRING, applet->priv->iid,
-			       0);
+			       NULL);
 
 
 	if (!ret) { /* FIXME */
@@ -1266,6 +1244,7 @@ panel_applet_class_init (PanelAppletClass *klass,
 	widget_class->size_allocate = panel_applet_size_allocate;
 	widget_class->expose_event = panel_applet_expose;
 	widget_class->focus = panel_applet_focus;
+	widget_class->realize = panel_applet_realize;
 
 	gobject_class->finalize = panel_applet_finalize;
 
@@ -1407,13 +1386,6 @@ panel_applet_setup (PanelApplet *applet)
 	priv->hierarchy_changed_id  = 0;
 }
 
-/**
- * panel_applet_new:
- *
- * Creates a new #PanelApplet.
- *
- * Return value: A #GtkWidget on success, %NULL on failure.
- */
 GtkWidget *
 panel_applet_new (void)
 {
@@ -1478,16 +1450,6 @@ panel_applet_all_controls_dead (void)
 		bonobo_main_quit ();
 }
 
-/**
- * panel_applet_factory_main_closure:
- * @iid: The bonobo-activation iid of the factory.
- * @closure: The factory callback closure.
- *
- * A generic 'main' routine for applets. This should not normally be
- * used directly because it is invoked by #PANEL_APPLET_BONOBO_FACTORY.
- *
- * Return value: 0 on success, 1 on failure.
- */
 int
 panel_applet_factory_main_closure (const gchar *iid,
 				   GType        applet_type,
@@ -1524,17 +1486,6 @@ panel_applet_factory_main_closure (const gchar *iid,
 	return retval;
 }
 
-/**
- * panel_applet_factory_main:
- * @iid: The bonobo-activation iid of the factory.
- * @callback: The factory callback.
- * @data: The factory user data pointer.
- *
- * A generic 'main' routine for applets. This should not normally be
- * used directly because it is invoked by #PANEL_APPLET_BONOBO_FACTORY.
- *
- * Return value: 0 on success, 1 on failure.
- */
 int
 panel_applet_factory_main (const gchar                 *iid,
 			   GType                        applet_type,
